-------------------------
-- Text Node prototype --
-------------------------

oplist = {nbsp=" ", amp="&", lt="<", gt=">", quot=""}

function fixop(op)
	local n = tonumber(op:sub(3,-2))
	if n then
		return string.uchar(n)
	else
		return oplist[op:sub(2,-2)] or ""
	end
end

TextNode = class()

function TextNode:init(parent, text)
	self.parent = parent
	self.text = text
	self.tagtype = "TextNode"
	self.childs = {}
end

function TextNode:paint(gc, x, y, fontstyle, fontsize, revx)
	gc:setFont("sansserif", fontstyle, fontsize)
	local words = self.text:split()
	local xpos, ypos = x, 0
	local space = gc:getStringWidth(" ")
	local sentence = words[1]
	local w = gc:getStringWidth(words[1])
	for i=2, #words do
		local word = words[i]:gsub("&[0-9a-zA-Z]+;", fixop)
		w = gc:getStringWidth(sentence .. " " .. word)
		if w + xpos < pW then
			sentence = sentence .. " " .. word
		else
			gc:drawString(sentence, xpos, y+ypos, "top")
			ypos = ypos + fontsize + 2
			sentence = word
			w = gc:getStringWidth(word)
			xpos=revx
		end
	end
	gc:drawString(sentence, xpos, y+ypos, "top")
	
	return w+space, ypos
end

-----------------------------
-- Font string manipulator --
-----------------------------

function setFont(str, t, b)
	local types = {}
	for i=1, #str do
		types[str:sub(i,i)] = true
	end
	types[t] = b
	local ff = (types.b and "b" or "") .. (types.i and "i" or "")
	return ff ~= "" and ff or "r"
end

---------------------------
-- HTML Object prototype --
---------------------------

HTMLobject = class()

function HTMLobject:init(parent, tagtype)
	self.parent = parent
	self.childs = {}
	self.tagtype=tagtype
end

function HTMLobject:paintChilds(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu, wuu, huu = 0,0,0,0
	for _, child in pairs(self.childs) do
		wuu, huu, d = child:paint(gc, x+wu, y+hu, fontstyle, fontsize, revx)
		wu = not d and wu+wuu or 0
		hu = hu + huu 
	end
	
	return wu, hu
end

function HTMLobject:paint(gc, x, y, fontstyle, fontsize, revx)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

---------------------------------------
-- Container for HTML object classes --
---------------------------------------
HTMLobjects = {}

setmetatable (HTMLobjects, {__index=function () return HTMLobject end})

-------------
-- <b> tag --
-------------
HTMLobjects.b = class(HTMLobject)
function HTMLobjects.b:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "b", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <i> tag --
-------------
HTMLobjects.i = class(HTMLobject)
function HTMLobjects.i:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "i", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <u> tag --
-------------
HTMLobjects.u = class(HTMLobject)
function HTMLobjects.u:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "u", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <h2> tag --
-------------
HTMLobjects.h2 = class(HTMLobject)
function HTMLobjects.h2:paint(gc, x, y, fontstyle, fontsize, revx)
	fontsize = fontsize + 2
	local wu, hu = self:paintChilds(gc, x, y , fontstyle, fontsize, revx)
	return 0, hu + fontsize + 6, true
end


-------------
-- <table> tag --
-------------
HTMLobjects.table = class(HTMLobject)
function HTMLobjects.table:paint(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu = self:paintChilds(gc, x, y , fontstyle, fontsize, revx)
	return 0, hu + fontsize + 2, true
end

-------------
-- <hr> tag --
-------------
HTMLobjects.hr = class(HTMLobject)
function HTMLobjects.hr:paint(gc, x, y, fontstyle, fontsize, revx)
	gc:fillRect(0, y + fontsize + 4 + fontsize/2, pW, 2)
	return 0, fontsize*2 + 4, true
end


-------------
-- <img> tag --
-------------
HTMLobjects.img = class(HTMLobject)
function HTMLobjects.img:paint(gc, x, y, fontstyle, fontsize, revx)
	return 0, -fontsize*2-2
end

-------------
-- <p> tag --
-------------
HTMLobjects.p = class(HTMLobject)
function HTMLobjects.p:paint(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu = self:paintChilds(gc, revx, (fontsize+2)*1.5 + y, fontstyle, fontsize, revx)
	return 0, hu + (fontsize+2)*3 or 0, true
end

HTMLobjects.pre = HTMLobjects.p


-------------
-- <br> tag --
-------------
HTMLobjects.br = class(HTMLobject)
function HTMLobjects.br:paint(gc, x, y, fontstyle, fontsize)
	return 0, fontsize+2, true
end

