-------------------------------
-- Start of Construction.lua --
-------------------------------
-------------------------
-- Construction Events --
-------------------------

pW = 318
pH = 212

function init()
	-- Nothing yet
end

on.construction = init
on.create       = init

function on.resize(w, h)
	pW = w
	pH = h
end
-----------------------------
-- End of Construction.lua --
-----------------------------

------------------------
-- Start of Tools.lua --
------------------------
-------------------------------
-- String function extension --
-------------------------------

--[[
* Str:devide(pattern)
* string.devide(Str, pattern)

> string : a general string
> pattern: Lua pattern

< Table of strings
--]]

function string:devide(pattern)
	local out = {}
	local sp, ep, ssp, eep = 0, 0, 0, 0
	local len = #self
	local outp = 0
 
	while sp<len do
		ssp, eep = self:find(pattern, ep+1)
		if not ssp then ssp=len+1 end
		if ssp-ep>1 then
			outp = outp + 1
			out[outp] = self:sub(ep+1, ssp-1)
		end
		if eep then
			outp = outp + 1
			out[outp] = self:sub(ssp, eep)
		end
		sp = ssp
		ep = eep
	end
 
	return out
end
----------------------
-- End of Tools.lua --
----------------------

-----------------------------
-- Start of nspirelink.lua --
-----------------------------
local pw	= getmetatable(platform.window)
function pw:invalidateAll()
	self:setFocus(false)
	self:setFocus(true)
end

function on.paint(gc)
	on.rpaint(gc)
	platform.withGC(on.draw)
end


-- Bit lib

function shiftLeft(int, n)                       -- int << n
	return int*2^n
end

function shiftRight(int, n)                      -- int >> n
	return math.floor(int/2^n)
end


--Base64 

function code1(x)
	if (x == '=') then 
		return ''
	end
	
    local r, f = '', x:byte()-128
	for i=6, 1, -1 do
		r = r .. (f%2^i - f%2^(i-1) > 0 and '1' or '0')
	end	
	
	return r;
end

function code2(x)
	if (#x ~= 8) then
		return ''
	end
	
    local c=0
    for i=1, 8 do
		c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) 
	end
    
    return string.char(c)
end


function b64dec(data)
    return data:gsub('.', code1):gsub('%d%d%d?%d?%d?%d?%d?%d?', code2):gsub("%z","")
end


--encode data

function encode(a,b)
	local n = shiftLeft(a:byte(), 8)   +   b:byte()       
	
	local r = shiftRight(n,11)                       -- 5b
	local n = n - shiftLeft(r, 11)
	
	local g = shiftRight(n, 5)                       -- 6b 
	local b = n - shiftLeft(g, 5)
	return shiftLeft(r, 19) + shiftLeft(g, 10) + shiftLeft(b, 3)
end


function drawEString(gc, str)
	local l = #str
	if l%2==1 then str = str .. " " l=l+1 end
	
	for i=1, l, 2 do
		local color = encode(str:sub(i,i), str:sub(i+1,i+1))
		gc:setColorRGB(color)
		gc:fillRect(math.floor(i/2), 0, 1, 1)
	end
	gc:setColorRGB(0xFFFFFF)
	gc:fillRect(math.floor(l/2), 0, 1, 1)
end


send_buffer = {}

function on.draw(gc)
	gc:setColorRGB(0xFFFFFF)
	gc:fillRect(0, 0, 1, 1)
	local l = #send_buffer
	if l>=1 then
		drawEString(gc, send_buffer[1])
	end
end

function on.backtabKey()
	table.remove(send_buffer, 1)
	platform.window:invalidateAll()
end

link_receiving = false
link_buffer = ""

function on.charIn(ch)
	local agr = (ch == "sin(")
	if agr then link_receiving = not link_receiving end
	
	if #ch == 2 and link_receiving then
		if ch == "\194\190" then
			ch = "="
		else
			ch = ch:sub(2,2)
		end
		
		link_buffer = link_buffer .. ch
	elseif not agr then
		on.rcharIn(ch)
	end

	if agr and not link_receiving then
		on.receive(b64dec(link_buffer))
		link_buffer = ""
	end
end



function link_send(data)
	table.insert(send_buffer, data)
	platform.window:invalidateAll()
end
---------------------------
-- End of nspirelink.lua --
---------------------------

------------------------------
-- Start of HTMLobjects.lua --
------------------------------
-------------------------
-- Text Node prototype --
-------------------------

oplist = {nbsp=" ", amp="&", lt="<", gt=">", quot=""}

function fixop(op)
	local n = tonumber(op:sub(3,-2))
	if n then
		return string.uchar(n)
	else
		return oplist[op:sub(2,-2)] or ""
	end
end

TextNode = class()

function TextNode:init(parent, text)
	self.parent = parent
	self.text = text
	self.tagtype = "TextNode"
	self.childs = {}
end

function TextNode:paint(gc, x, y, fontstyle, fontsize, revx)
	gc:setFont("sansserif", fontstyle, fontsize)
	local words = self.text:split()
	local xpos, ypos = x, 0
	local space = gc:getStringWidth(" ")
	local sentence = words[1]
	local w = gc:getStringWidth(words[1])
	for i=2, #words do
		local word = words[i]:gsub("&[0-9a-zA-Z]+;", fixop)
		w = gc:getStringWidth(sentence .. " " .. word)
		if w + xpos < pW then
			sentence = sentence .. " " .. word
		else
			gc:drawString(sentence, xpos, y+ypos, "top")
			ypos = ypos + fontsize + 2
			sentence = word
			w = gc:getStringWidth(word)
			xpos=revx
		end
	end
	gc:drawString(sentence, xpos, y+ypos, "top")
	
	return w+space, ypos
end

-----------------------------
-- Font string manipulator --
-----------------------------

function setFont(str, t, b)
	local types = {}
	for i=1, #str do
		types[str:sub(i,i)] = true
	end
	types[t] = b
	local ff = (types.b and "b" or "") .. (types.i and "i" or "")
	return ff ~= "" and ff or "r"
end

---------------------------
-- HTML Object prototype --
---------------------------

HTMLobject = class()

function HTMLobject:init(parent, tagtype)
	self.parent = parent
	self.childs = {}
	self.tagtype=tagtype
end

function HTMLobject:paintChilds(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu, wuu, huu = 0,0,0,0
	for _, child in pairs(self.childs) do
		wuu, huu, d = child:paint(gc, x+wu, y+hu, fontstyle, fontsize, revx)
		wu = not d and wu+wuu or 0
		hu = hu + huu 
	end
	
	return wu, hu
end

function HTMLobject:paint(gc, x, y, fontstyle, fontsize, revx)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

---------------------------------------
-- Container for HTML object classes --
---------------------------------------
HTMLobjects = {}

setmetatable (HTMLobjects, {__index=function () return HTMLobject end})

-------------
-- <b> tag --
-------------
HTMLobjects.b = class(HTMLobject)
function HTMLobjects.b:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "b", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <i> tag --
-------------
HTMLobjects.i = class(HTMLobject)
function HTMLobjects.i:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "i", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <u> tag --
-------------
HTMLobjects.u = class(HTMLobject)
function HTMLobjects.u:paint(gc, x, y, fontstyle, fontsize, revx)
	fontstyle = setFont(fontstyle, "u", true)
	return self:paintChilds(gc, x, y, fontstyle, fontsize, revx)
end

-------------
-- <h2> tag --
-------------
HTMLobjects.h2 = class(HTMLobject)
function HTMLobjects.h2:paint(gc, x, y, fontstyle, fontsize, revx)
	fontsize = fontsize + 2
	local wu, hu = self:paintChilds(gc, x, y , fontstyle, fontsize, revx)
	return 0, hu + fontsize + 6, true
end


-------------
-- <table> tag --
-------------
HTMLobjects.table = class(HTMLobject)
function HTMLobjects.table:paint(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu = self:paintChilds(gc, x, y , fontstyle, fontsize, revx)
	return 0, hu + fontsize + 2, true
end

-------------
-- <hr> tag --
-------------
HTMLobjects.hr = class(HTMLobject)
function HTMLobjects.hr:paint(gc, x, y, fontstyle, fontsize, revx)
	gc:fillRect(0, y + fontsize + 4 + fontsize/2, pW, 2)
	return 0, fontsize*2 + 4, true
end


-------------
-- <img> tag --
-------------
HTMLobjects.img = class(HTMLobject)
function HTMLobjects.img:paint(gc, x, y, fontstyle, fontsize, revx)
	return 0, -fontsize*2-2
end

-------------
-- <p> tag --
-------------
HTMLobjects.p = class(HTMLobject)
function HTMLobjects.p:paint(gc, x, y, fontstyle, fontsize, revx)
	local wu, hu = self:paintChilds(gc, revx, (fontsize+2)*1.5 + y, fontstyle, fontsize, revx)
	return 0, hu + (fontsize+2)*3 or 0, true
end

HTMLobjects.pre = HTMLobjects.p


-------------
-- <br> tag --
-------------
HTMLobjects.br = class(HTMLobject)
function HTMLobjects.br:paint(gc, x, y, fontstyle, fontsize)
	return 0, fontsize+2, true
end

----------------------------
-- End of HTMLobjects.lua --
----------------------------

-----------------------------
-- Start of HTMLparser.lua --
-----------------------------
-----------------
-- HTML parser --
-----------------

HTMLdocument = class()

HTMLdocument.tagtype = {html=true, head=true, body=true, title=true, b=true, i=true, u=true, br=false, hr=false, h1=true, h2=true, h3=true, h4=true, h5=true, h6=true, pre=true}

function HTMLdocument:init(src)
	self.source = src
	self.htmltree = HTMLobject(nil, "root")
	self.taglist = {}
end

function isTag(block)
	if block:sub(1,1) == "<" and block:sub(-1,-1) == ">" then
		return block:sub(2,-2)
	else
		return false
	end
end

function HTMLdocument:parse()
	local div = self.source:devide("<[^<>]*>")
	local ctag = self.htmltree
	
	
	for _, block in ipairs(div) do
		local data = isTag(block)
		
		if data then
			local tag = data:split(" ")[1]:lower()
			if tag:sub(-1,-1) == "/" then tag = tag:sub(1,-2) end
			
			if tag:sub(1,1) == "/" then
				if HTMLdocument.tagtype[tag:sub(2,-1)] and ctag.parent then
					ctag = ctag.parent
				end
			else
				
				local tagtable = HTMLobjects[tag](ctag, tag)
				table.insert(ctag.childs, tagtable)
				
				self.taglist[tag] = self.taglist[tag] or {}
				table.insert(self.taglist[tag], tagtable)
				if HTMLdocument.tagtype[tag] then ctag=tagtable end
			end
		else
			local s, e = block:find("[\r\n]+")
			if not (s == 1 and e == #block) then
				if ctag.tagtype ~= "pre" then block=block:gsub("%s+", " "):gsub("^%s","") end
				if block ~= "" then
					table.insert(ctag.childs, TextNode(ctag, block))
				end
			end
		end
	end
	
	return self.htmltree
end

function HTMLdocument:getElementsByTagName(tag)
	return self.taglist[tag:lower()] or {}
end

function HTMLdocument:paint(gc)
	gc:setColorRGB(0,0,0)
	gc:fillRect(0,0,pW,pH)
	gc:setColorRGB(255,255,255)
	self:getElementsByTagName("body")[1]:paint(gc, 0, 0, "r", 9, 0)
end
---------------------------
-- End of HTMLparser.lua --
---------------------------


msg = ""
function on.rpaint(gc)
	gc:setColorRGB(0)
	if msg then
		gc:drawString("Enter url:",10,10,"top")
		gc:drawString(msg,10,20,"top")
	end
	if doc then
		doc:paint(gc)
	end
end

function printtag(tag, l)
	l=l or 0
	print(string.rep("  ", l) .. (tag.tagtype or "huhtag"))
	for _, child in ipairs(tag.childs) do
		printtag(child, l+1)
	end
end

--printtag(doc.htmltree)

function on.rcharIn(ch)
	if msg then
		msg = msg .. ch
	end
	platform.window:invalidate()
end

function on.enterKey()
	if msg then
		link_send("HTTPFILE GET " .. msg)
		platform.window:invalidate()
		msg="Receiving data..."
	end
end

function on.receive(data)
	if data == "HF 1" then
		msg=nil
		webdata = var.recall("cache\\cache")
		doc=HTMLdocument(webdata)
		doc:parse()
		platform.window:invalidate()
	end
end

