local pw	= getmetatable(platform.window)
function pw:invalidateAll()
	self:setFocus(false)
	self:setFocus(true)
end

function on.paint(gc)
	on.rpaint(gc)
	platform.withGC(on.draw)
end


-- Bit lib

function shiftLeft(int, n)                       -- int << n
	return int*2^n
end

function shiftRight(int, n)                      -- int >> n
	return math.floor(int/2^n)
end


--Base64 

function code1(x)
	if (x == '=') then 
		return ''
	end
	
    local r, f = '', x:byte()-128
	for i=6, 1, -1 do
		r = r .. (f%2^i - f%2^(i-1) > 0 and '1' or '0')
	end	
	
	return r;
end

function code2(x)
	if (#x ~= 8) then
		return ''
	end
	
    local c=0
    for i=1, 8 do
		c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) 
	end
    
    return string.char(c)
end


function b64dec(data)
    return data:gsub('.', code1):gsub('%d%d%d?%d?%d?%d?%d?%d?', code2):gsub("%z","")
end


--encode data

function encode(a,b)
	local n = shiftLeft(a:byte(), 8)   +   b:byte()       
	
	local r = shiftRight(n,11)                       -- 5b
	local n = n - shiftLeft(r, 11)
	
	local g = shiftRight(n, 5)                       -- 6b 
	local b = n - shiftLeft(g, 5)
	return shiftLeft(r, 19) + shiftLeft(g, 10) + shiftLeft(b, 3)
end


function drawEString(gc, str)
	local l = #str
	if l%2==1 then str = str .. " " l=l+1 end
	
	for i=1, l, 2 do
		local color = encode(str:sub(i,i), str:sub(i+1,i+1))
		gc:setColorRGB(color)
		gc:fillRect(math.floor(i/2), 0, 1, 1)
	end
	gc:setColorRGB(0xFFFFFF)
	gc:fillRect(math.floor(l/2), 0, 1, 1)
end


send_buffer = {}

function on.draw(gc)
	gc:setColorRGB(0xFFFFFF)
	gc:fillRect(0, 0, 1, 1)
	local l = #send_buffer
	if l>=1 then
		drawEString(gc, send_buffer[1])
	end
end

function on.backtabKey()
	table.remove(send_buffer, 1)
	platform.window:invalidateAll()
end

link_receiving = false
link_buffer = ""

function on.charIn(ch)
	local agr = (ch == "sin(")
	if agr then link_receiving = not link_receiving end
	
	if #ch == 2 and link_receiving then
		if ch == "\194\190" then
			ch = "="
		else
			ch = ch:sub(2,2)
		end
		
		link_buffer = link_buffer .. ch
	elseif not agr then
		on.rcharIn(ch)
	end

	if agr and not link_receiving then
		on.receive(b64dec(link_buffer))
		link_buffer = ""
	end
end



function link_send(data)
	table.insert(send_buffer, data)
	platform.window:invalidateAll()
end
